import React, { useRef, useEffect } from "react";
/* global google */

const MapComponent = React.memo(({
  onMapClick
}) => {
  const mapDivRef = useRef(null);
  useEffect(() => {
    const googleMap = new google.maps.Map(mapDivRef.current, {
      center: { lat: 25, lng: -25 },
      zoom: 0,
      disableDefaultUI: true,
      fullscreenControl: true,
    });
    console.log("Created Map");
    if (onMapClick) {
      console.log("Adding click listener");
      googleMap.addListener("click", ({latLng}) => onMapClick(googleMap, latLng));
    }
  });

  return (
    <div 
      style={{ height: "100%" }}
      ref={mapDivRef}
    />
  );
}, () => true);

export default ({
  onMarkerMoved,
}) => {
  // avoid re-rendering by using let (similar to instance field)
  let marker = null;
  return <MapComponent onMapClick={(map, position) => {
    if (marker) {
      marker.setMap(null);
    }
    marker = new google.maps.Marker({ map, position });
    if (onMarkerMoved) {
      onMarkerMoved({ lat: position.lat(), lng: position.lng() });
    }
  }}/>
};
